@@ -20,6 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <string.h>
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-decl.h>
 #include <libgda/gda-data-model-query.h>
@@ -45,6 +46,8 @@ struct _GdaDataModelQueryPrivate {
 	gboolean          defer_refresh;
 	gboolean          refresh_pending;
 	GSList           *columns;
+
+	gboolean          auto_compute;
 };
 
 /* properties */
@@ -54,7 +57,8 @@ enum
         PROP_SEL_QUERY,
 	PROP_INS_QUERY,
 	PROP_UPD_QUERY,
-	PROP_DEL_QUERY
+	PROP_DEL_QUERY,
+	PROP_AUTO_COMPUTE_QUERIES
 };
 
 enum 
@@ -86,11 +90,13 @@ static gint                 gda_data_model_query_get_n_rows      (GdaDataModel *
 static gint                 gda_data_model_query_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_model_query_describe_column (GdaDataModel *model, gint col);
 static guint                gda_data_model_query_get_access_flags(GdaDataModel *model);
-static const GdaValue      *gda_data_model_query_get_value_at    (GdaDataModel *model, gint col, gint row);
+static const GValue        *gda_data_model_query_get_value_at    (GdaDataModel *model, gint col, gint row);
 static guint                gda_data_model_query_get_attributes_at (GdaDataModel *model, gint col, gint row);
 
+static GdaDataModelIter    *gda_data_model_query_create_iter     (GdaDataModel *model);
+
 static gboolean             gda_data_model_query_set_value_at    (GdaDataModel *model, gint col, gint row, 
-								   const GdaValue *value, GError **error);
+								   const GValue *value, GError **error);
 static gboolean             gda_data_model_query_set_values      (GdaDataModel *model, gint row, 
 								   GList *values, GError **error);
 static gint                 gda_data_model_query_append_values   (GdaDataModel *model, const GList *values, GError **error);
@@ -98,12 +104,14 @@ static gint                 gda_data_model_query_append_row      (GdaDataModel *
 static gboolean             gda_data_model_query_remove_row      (GdaDataModel *model, gint row, 
 								  GError **error);
 static void                 gda_data_model_query_send_hint       (GdaDataModel *model, GdaDataModelHint hint, 
-								  const GdaValue *hint_value);
+								  const GValue *hint_value);
 
 static void create_columns (GdaDataModelQuery *model);
 static void to_be_destroyed_query_cb (GdaQuery *query, GdaDataModelQuery *model);
 static void param_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GdaDataModelQuery *model);
 
+static gboolean auto_compute_modif_queries (GdaDataModelQuery *model, GError **error);
+
 #ifdef GDA_DEBUG
 static void gda_data_model_query_dump (GdaDataModelQuery *select, guint offset);
 #endif
@@ -177,6 +185,13 @@ gda_data_model_query_class_init (GdaDataModelQueryClass *klass)
 							       "DELETE Query to be executed to remove data",
 							       G_PARAM_READABLE | G_PARAM_WRITABLE));
 
+	g_object_class_install_property (object_class, PROP_AUTO_COMPUTE_QUERIES,
+					 g_param_spec_boolean ("auto_compute", "Auto compute modifications query", 
+							       "Set to TRUE to make the object automatically "
+							       "compute modification queries from a SELECT query", 
+							       FALSE,
+							       G_PARAM_READABLE | G_PARAM_WRITABLE));
+
 	/* virtual functions */
 	object_class->dispose = gda_data_model_query_dispose;
 	object_class->finalize = gda_data_model_query_finalize;
@@ -198,7 +213,7 @@ gda_data_model_query_data_model_init (GdaDataModelClass *iface)
 	iface->i_get_value_at = gda_data_model_query_get_value_at;
 	iface->i_get_attributes_at = gda_data_model_query_get_attributes_at;
 
-	iface->i_create_iter = NULL; 
+	iface->i_create_iter = gda_data_model_query_create_iter; 
 	iface->i_iter_at_row = NULL;
 	iface->i_iter_next = NULL;
 	iface->i_iter_prev = NULL;
@@ -227,6 +242,8 @@ gda_data_model_query_init (GdaDataModelQuery *model, GdaDataModelQueryClass *kla
 	/* model refreshing is performed as soon as any modification is done */
 	model->priv->defer_refresh = FALSE;
 	model->priv->refresh_pending = FALSE;
+
+	model->priv->auto_compute = FALSE;
 }
 
 static void
@@ -355,26 +372,39 @@ gda_data_model_query_set_property (GObject *object,
 				g_object_ref (model->priv->queries[qindex]);
 				g_signal_connect (model->priv->queries[qindex], "to_be_destroyed",
 						  G_CALLBACK (to_be_destroyed_query_cb), model);
-
-				model->priv->params[qindex] = gda_entity_get_param_list (GDA_ENTITY (model->priv->queries[qindex]));
+				
+				model->priv->params[qindex] = gda_query_get_parameter_list (model->priv->queries[qindex]);
 
 				if (qindex == SEL_QUERY) {
 					/* SELECT query */
+					GSList *targets, *tlist;
+
+					targets = gda_query_get_targets (model->priv->queries[qindex]);
+					tlist = targets;
+					while (tlist) {
+						g_slist_free (gda_query_expand_all_field (model->priv->queries[qindex],
+											  GDA_QUERY_TARGET (tlist->data)));
+						tlist = g_slist_next (tlist);
+					}
+					g_slist_free (targets);
+
 					g_signal_connect (model->priv->queries[qindex], "changed",
 							  G_CALLBACK (query_changed_cb), model);
 
 					if (model->priv->params[qindex])
 						g_signal_connect (model->priv->params[qindex], "param_changed",
 								  G_CALLBACK (param_changed_cb), model);
+					if (model->priv->auto_compute)
+						auto_compute_modif_queries (model, NULL);
 				}
 				else {
 					/* other queries: for all the parameters in the param list, 
-					 * if some have a name like "_<num>", then they will be filled with
-					 * the value at the <num>th column before being run, or, if the name is
+					 * if some have a name like "[+-]<num>", then they will be filled with
+					 * the value at the new/old <num>th column before being run, or, if the name is
 					 * the same as a parameter for the SELECT query, then bind them to that parameter */
 					gint num;
 
-					if (model->priv->params [qindex] && model->priv->params [SEL_QUERY]) {
+					if (model->priv->params [qindex]) {
 						GSList *params = model->priv->params [qindex]->parameters;
 						while (params) {
 							const gchar *pname = gda_object_get_name (GDA_OBJECT (params->data));
@@ -389,9 +419,15 @@ gda_data_model_query_set_property (GObject *object,
 											   GINT_TO_POINTER (num + 1));
 								g_object_set_data ((GObject*) params->data, "_num",
 										   GINT_TO_POINTER (num + 1));
+								GdaColumn *col;
+								col = gda_data_model_describe_column (model, num);
+								if (col) 
+									gda_parameter_set_not_null 
+										((GdaParameter *)(params->data),
+										 !gda_column_get_allow_null (col));
 							}
 							else {
-								if (pname) {
+								if (pname && model->priv->params [SEL_QUERY]) {
 									GdaParameter *bind_to;
 									bind_to = gda_parameter_list_find_param 
 										(model->priv->params [SEL_QUERY],
@@ -407,6 +443,9 @@ gda_data_model_query_set_property (GObject *object,
 				}
 			}
 			break;
+		case PROP_AUTO_COMPUTE_QUERIES:
+			model->priv->auto_compute = g_value_get_boolean (value);
+			break;
 		default:
 			g_assert_not_reached ();
 			break;
@@ -432,6 +471,9 @@ gda_data_model_query_get_property (GObject *object,
 		case PROP_DEL_QUERY:
 			g_value_set_pointer (value, model->priv->queries[qindex]);
 			break;
+		case PROP_AUTO_COMPUTE_QUERIES:
+			g_value_set_boolean (value, model->priv->auto_compute);
+			break;
 		default:
 			g_assert_not_reached ();
 			break;
@@ -489,7 +531,7 @@ param_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GdaDataModel
 			g_object_unref (model->priv->data);
 			model->priv->data = NULL;
 		}
-		gda_data_model_changed ((GdaDataModel *) model);
+		gda_data_model_signal_emit_changed ((GdaDataModel *) model);
 	}
 }
 
@@ -509,23 +551,24 @@ gda_data_model_query_new (GdaQuery *query)
 
 	g_return_val_if_fail (GDA_IS_QUERY (query), NULL);
 
-	model = g_object_new (GDA_TYPE_DATA_MODEL_QUERY, "dict", gda_object_get_dict (GDA_OBJECT (query)),
+	model = g_object_new (GDA_TYPE_DATA_MODEL_QUERY, 
+			      "dict", gda_object_get_dict (GDA_OBJECT (query)),
 			      "query", query, NULL);
 
 	return GDA_DATA_MODEL (model);
 }
 
 /**
- * gda_data_model_query_get_param_list
+ * gda_data_model_query_get_parameter_list
  * @model: a #GdaDataModelQuery data model
  *
  * If some parameters are required to execute the SELECT query used in the @model data model, then
- * creates a new #GdaParameterList; otherwise does nothing and returns %NULL.
+ * returns the #GdaParameterList used; otherwise does nothing and returns %NULL.
  *
- * Returns: a new #GdaParameterList object, or %NULL
+ * Returns: a #GdaParameterList object, or %NULL
  */
 GdaParameterList *
-gda_data_model_query_get_param_list (GdaDataModelQuery *model)
+gda_data_model_query_get_parameter_list (GdaDataModelQuery *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_QUERY (model), NULL);
 	g_return_val_if_fail (model->priv, NULL);
@@ -540,15 +583,11 @@ gda_data_model_query_get_param_list (GdaDataModelQuery *model)
  *
  * (Re)-runs the SELECT query to update the contents of @model
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_data_model_query_refresh (GdaDataModelQuery *model, GError **error)
 {
-	gchar *sql;
-	GdaCommand *cmd;
-	GdaConnection *cnc;
-
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_QUERY (model), FALSE);
 	g_return_val_if_fail (model->priv, FALSE);
 
@@ -572,53 +611,17 @@ gda_data_model_query_refresh (GdaDataModelQuery *model, GError **error)
 		return FALSE;
 	}
 
-	cnc = gda_dict_get_connection (gda_object_get_dict (GDA_OBJECT (model)));
-	if (!cnc) {
-		g_set_error (&model->priv->refresh_error, 0, 0,
-			     _("No connection specified"));
-		if (error) 
-			*error = g_error_copy (model->priv->refresh_error);
-		return FALSE;
-	}
-
-	if (!gda_connection_is_opened (cnc)) {
-		g_set_error (&model->priv->refresh_error, 0, 0,
-			     _("Connection is not opened"));
-		if (error) 
-			*error = g_error_copy (model->priv->refresh_error);
-		return FALSE;
-	}
-
-	sql = gda_renderer_render_as_sql (GDA_RENDERER (model->priv->queries[SEL_QUERY]), model->priv->params [SEL_QUERY],
-					  0, &model->priv->refresh_error);
-	if (!sql) {
+	model->priv->data = gda_query_execute (model->priv->queries[SEL_QUERY], model->priv->params [SEL_QUERY],
+					       TRUE, &model->priv->refresh_error);
+	if (!model->priv->data || !GDA_IS_DATA_MODEL (model->priv->data)) {
+		model->priv->data = NULL;
 		g_assert (model->priv->refresh_error);
 		if (error) 
 			*error = g_error_copy (model->priv->refresh_error);
 		return FALSE;
 	}
 
-	cmd = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
-        model->priv->data = gda_connection_execute_single_command (cnc, cmd, NULL, &model->priv->refresh_error);
-	gda_command_free (cmd);
-	if (!model->priv->data) {
-		g_assert (model->priv->refresh_error);
-		if (error) 
-			*error = g_error_copy (model->priv->refresh_error);
-	}
-
-#ifdef GDA_DEBUG_NO
-	{
-		g_print ("GdaDataModelQuery refresh:\nSQL= %s\n", sql);
-		if (model->priv->data) 
-			gda_data_model_dump (model, stdout);
-		else
-			g_print ("\t=> error\n");
-	}
-#endif
-	g_free (sql);
-
-	gda_data_model_changed ((GdaDataModel *) model);
+	gda_data_model_signal_emit_changed ((GdaDataModel *) model);
 	return model->priv->data ? TRUE : FALSE;
 }
 
@@ -632,10 +635,19 @@ gda_data_model_query_refresh (GdaDataModelQuery *model, GError **error)
  * to the dataset in the database.
  *
  * The provided query (the @query SQL) must be either a INSERT, UPDATE or DELETE query. It can contain
- * parameters, and the parameters named '_&lt;num&gt;' will be replaced with the value of the 
- * &lt;num&gt;th column in @model's data set when run.
+ * parameters, and the parameters named '[+-]&lt;num&gt;' will be replaced when the query is run:
+ * <itemizedlist>
+ *   <listitem><para>a parameter named +&lt;num&gt; will take the new value set at the 
+ *                   &lt;num&gt;th column in @model</para></listitem>
+ *   <listitem><para>a parameter named -&lt;num&gt; will take the old value set at the
+ *                   &lt;num&gt;th column in @model</para></listitem>
+ * </itemizedlist>
+ * Please note that the "+0" and "-0" parameters names are valid and will respectively 
+ * take the new and old values of the first column of @model.
  *
- * Returns: TRUE if no error occured.
+ * Examples of queries are: "INSERT INTO orders (customer, creation_date, delivery_before, delivery_date) VALUES (## [:name="Customer" :type="integer"], date('now'), ## [:name="+2" :type="date" :nullok="TRUE"], NULL)", "DELETE FROM orders WHERE id = ## [:name="-0" :type="integer"]" and "UPDATE orders set id=## [:name="+0" :type="integer"], delivery_before=## [:name="+2" :type="date" :nullok="TRUE"], delivery_date=## [:name="+3" :type="date" :nullok="TRUE"] WHERE id=## [:name="-0" :type="integer"]"
+ *
+ * Returns: TRUE if no error occurred.
  */
 gboolean
 gda_data_model_query_set_modification_query (GdaDataModelQuery *model, const gchar *query, GError **error)
@@ -646,7 +658,7 @@ gda_data_model_query_set_modification_query (GdaDataModelQuery *model, const gch
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_QUERY (model), FALSE);
 	g_return_val_if_fail (model->priv, FALSE);
 
-	aq = GDA_QUERY (gda_query_new_from_sql (gda_object_get_dict ((GdaObject *) model), query, NULL));
+	aq = gda_query_new_from_sql (gda_object_get_dict ((GdaObject *) model), query, NULL);
 	if (gda_query_is_insert_query (aq)) {
 		g_object_set (model, "insert_query", aq, NULL);
 		done = TRUE;
@@ -671,12 +683,38 @@ gda_data_model_query_set_modification_query (GdaDataModelQuery *model, const gch
 	return TRUE;
 }
 
+/*
+ * auto_compute_modif_queries
+ *
+ * Try to compute the INSERT, DELETE and UPDATE queries from a SELECT query
+ */
+static gboolean
+auto_compute_modif_queries (GdaDataModelQuery *model, GError **error)
+{
+	return FALSE;
+}
 
 #ifdef GDA_DEBUG
 static void
 gda_data_model_query_dump (GdaDataModelQuery *select, guint offset)
 {
-	TO_IMPLEMENT;
+	gchar *str;
+
+	g_return_if_fail (GDA_IS_DATA_MODEL_QUERY (select));
+	
+        /* string for the offset */
+        str = g_new (gchar, offset+1);
+	memset (str, ' ', offset);
+        str[offset] = 0;
+	g_print ("%s" D_COL_H1 "GdaDataModelQuery %p" D_COL_NOR "\n", str, select);
+
+	if (select->priv->queries [SEL_QUERY])
+		gda_object_dump (GDA_OBJECT (select->priv->queries [SEL_QUERY]), offset + 5);
+
+	if (select->priv->params [SEL_QUERY]) 
+		gda_object_dump (GDA_OBJECT (select->priv->params [SEL_QUERY]), offset+5);
+		
+	g_free (str);
 }
 #endif
 
@@ -728,55 +766,40 @@ create_columns (GdaDataModelQuery *model)
 	if (! model->priv->queries[SEL_QUERY])
 		return;
 
-	if (model->priv->data) {
-		/* copy model->priv->data's columns */
-		gint i, nb_cols;
-
-		nb_cols = gda_data_model_get_n_columns (model->priv->data);
-		for (i = 0; i < nb_cols; i++) {
-			GdaColumn *orig, *col;
-			
-			orig = gda_data_model_describe_column (model->priv->data, i);
-			col = gda_column_copy (orig);
-			gda_column_set_position (col, i);
-			model->priv->columns = g_slist_append (model->priv->columns, col);
-		}
-	}
-	else {
+	gda_referer_activate (GDA_REFERER (model->priv->queries[SEL_QUERY]));	
+	if (gda_referer_is_active (GDA_REFERER (model->priv->queries[SEL_QUERY]))) {
 		GSList *fields, *list;
 		gboolean allok = TRUE;
 
-		gda_referer_activate (GDA_REFERER (model->priv->queries[SEL_QUERY]));
-		if (! gda_referer_is_active (GDA_REFERER (model->priv->queries[SEL_QUERY])))
-			return;
-		
 		fields = gda_entity_get_fields (GDA_ENTITY (model->priv->queries[SEL_QUERY]));
 		list = fields;
 		while (list && allok) {
-			if (gda_entity_field_get_gda_type (GDA_ENTITY_FIELD (list->data)) == GDA_VALUE_TYPE_UNKNOWN)
+			if (gda_entity_field_get_g_type (GDA_ENTITY_FIELD (list->data)) == G_TYPE_INVALID) {
 				allok = FALSE;
+				g_warning (_("Can't determine the GType for field '%s', please fill a bug report"), 
+					   gda_object_get_name (GDA_OBJECT (list->data)));
+			}
 			list = g_slist_next (list);
 		}
-		if (! allok)
+		if (! allok) 
 			return;
 
 		list = fields;
 		while (list) {
 			GdaColumn *col;
 			GdaEntityField *field = (GdaEntityField *) list->data;
-			
 
 			col = gda_column_new ();
 			gda_column_set_name (col, gda_object_get_name (GDA_OBJECT (field)));
 			gda_column_set_title (col, gda_object_get_name (GDA_OBJECT (field)));
-			gda_column_set_gda_type (col, gda_entity_field_get_gda_type (field));
+			gda_column_set_g_type (col, gda_entity_field_get_g_type (field));
 			if (GDA_IS_QUERY_FIELD_FIELD (field)) {
 				GdaEntityField *ref_field;
 
 				ref_field = gda_query_field_field_get_ref_field (GDA_QUERY_FIELD_FIELD (field));
 				if (GDA_IS_DICT_FIELD (ref_field)) {
 					GdaEntity *table;
-					const GdaValue *value;
+					const GValue *value;
 					gda_column_set_defined_size (col, gda_dict_field_get_length ((GdaDictField*) ref_field));
 					table = gda_entity_field_get_entity (ref_field);
 					gda_column_set_table (col, gda_object_get_name (GDA_OBJECT (table)));
@@ -794,6 +817,26 @@ create_columns (GdaDataModelQuery *model)
 
 		g_slist_free (fields);
 	}
+	else {
+		if (model->priv->data) {
+			/* copy model->priv->data's columns */
+			gint i, nb_cols;
+			
+			nb_cols = gda_data_model_get_n_columns (model->priv->data);
+			for (i = 0; i < nb_cols; i++) {
+				GdaColumn *orig, *col;
+				
+				orig = gda_data_model_describe_column (model->priv->data, i);
+				col = gda_column_copy (orig);
+				gda_column_set_position (col, i);
+				model->priv->columns = g_slist_append (model->priv->columns, col);
+			}
+		}
+		else
+			g_warning (_("Can't compute the list of columns because the "
+				   "SELECT query is not active (you can use a dictionary to correct this) "
+				   "and the model does not contain any data"));
+	}
 }
 
 static GdaColumn *
@@ -830,10 +873,11 @@ gda_data_model_query_get_access_flags (GdaDataModel *model)
 	if (selmodel->priv->data) {
 		gint i;
 		for (i = INS_QUERY; i <= DEL_QUERY; i++) {
+			gboolean allok = TRUE;
+
 			if (selmodel->priv->params [i]) {
 				/* if all the parameters which are not named _[0-9]* are valid, 
 				 * then we grant the corresponding flag */
-				gboolean allok = TRUE;
 				GSList *params = selmodel->priv->params [i]->parameters;
 				while (params && allok) {
 					gint num;
@@ -848,23 +892,28 @@ gda_data_model_query_get_access_flags (GdaDataModel *model)
 					params = g_slist_next (params);
 				}
 				
-				if (allok) {
-					switch (i) {
-					case INS_QUERY:
-						flags |= GDA_DATA_MODEL_ACCESS_INSERT;
-						/* g_print ("INS flag\n"); */
-						break;
-					case UPD_QUERY:
-						flags |= GDA_DATA_MODEL_ACCESS_UPDATE;
-						/* g_print ("UPD flag\n"); */
-						break;
-					case DEL_QUERY:
-						flags |= GDA_DATA_MODEL_ACCESS_DELETE;
-						/* g_print ("DEL flag\n"); */
-						break;
-					default:
-						g_assert_not_reached ();
-					}
+			}
+			else {
+				if (!selmodel->priv->queries [i])
+					allok = FALSE;
+			}
+
+			if (allok) {
+				switch (i) {
+				case INS_QUERY:
+					flags |= GDA_DATA_MODEL_ACCESS_INSERT;
+					/* g_print ("INS flag\n"); */
+					break;
+				case UPD_QUERY:
+					flags |= GDA_DATA_MODEL_ACCESS_UPDATE;
+					/* g_print ("UPD flag\n"); */
+					break;
+				case DEL_QUERY:
+					flags |= GDA_DATA_MODEL_ACCESS_DELETE;
+					/* g_print ("DEL flag\n"); */
+					break;
+				default:
+					g_assert_not_reached ();
 				}
 			}
 		}
@@ -873,7 +922,7 @@ gda_data_model_query_get_access_flags (GdaDataModel *model)
 	return flags;
 }
 
-static const GdaValue *
+static const GValue *
 gda_data_model_query_get_value_at (GdaDataModel *model, gint col, gint row)
 {
 	GdaDataModelQuery *selmodel;
@@ -895,7 +944,7 @@ gda_data_model_query_get_attributes_at (GdaDataModel *model, gint col, gint row)
 {
 	guint flags = 0;
 	GdaDataModelQuery *selmodel;
-	gboolean used = FALSE;
+	GdaParameter *p_used = NULL;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_QUERY (model), 0);
 	selmodel = (GdaDataModelQuery *) model;
@@ -906,65 +955,84 @@ gda_data_model_query_get_attributes_at (GdaDataModel *model, gint col, gint row)
 
 	if ((row >= 0) && selmodel->priv->queries[UPD_QUERY] && selmodel->priv->params[UPD_QUERY]) {
 		GSList *params = selmodel->priv->params[UPD_QUERY]->parameters;
-		while (params && !used) {
+		while (params && !p_used) {
 			if (GPOINTER_TO_INT (g_object_get_data ((GObject*) params->data, "+num")) - 1 == col)
-				used = TRUE;
+				p_used = (GdaParameter *) params->data;
 			params = params->next;
 		}
 	}
 
 	if ((row < 0) && selmodel->priv->queries[INS_QUERY] && selmodel->priv->params[INS_QUERY]) {
 		GSList *params = selmodel->priv->params[INS_QUERY]->parameters;
-		while (params && !used) {
+		while (params && !p_used) {
 			if (GPOINTER_TO_INT (g_object_get_data ((GObject*) params->data, "+num")) - 1 == col)
-				used = TRUE;
+				p_used = (GdaParameter *) params->data;
 			params = params->next;
 		}
 	}
-	
-	if (!used)
+
+	if (!p_used)
 		flags |= GDA_VALUE_ATTR_NO_MODIF;
+	else {
+		flags &= ~GDA_VALUE_ATTR_NO_MODIF;
+		flags &= ~GDA_VALUE_ATTR_CAN_BE_NULL;
+		flags &= ~GDA_VALUE_ATTR_CAN_BE_DEFAULT;
+		if (! gda_parameter_get_not_null (p_used))
+			flags |= GDA_VALUE_ATTR_CAN_BE_NULL;
+		if (gda_parameter_get_default_value (p_used))
+			flags |= GDA_VALUE_ATTR_CAN_BE_DEFAULT;
+	}
+
+	/*g_print ("%d,%d: %d\n", col, row, flags);*/
 
 	return flags;
 }
 
+static GdaDataModelIter *
+gda_data_model_query_create_iter (GdaDataModel *model)
+{
+	GdaDataModelIter *iter;
+
+	iter = (GdaDataModelIter *) g_object_new (GDA_TYPE_DATA_MODEL_ITER, 
+						  "dict", gda_object_get_dict (GDA_OBJECT (model)), 
+						  "data_model", model, NULL);
+	/* set the "entry_plugin" for all the parameters depending on the SELECT query field */
+	if (gda_query_is_select_query (GDA_DATA_MODEL_QUERY (model)->priv->queries[SEL_QUERY])) {
+		GSList *list, *fields;
+		GSList *params;
+
+		fields = gda_entity_get_fields (GDA_ENTITY (GDA_DATA_MODEL_QUERY (model)->priv->queries[SEL_QUERY]));
+		params = GDA_PARAMETER_LIST (iter)->parameters;
+
+		for (list = fields; list && params; list = list->next, params = params->next) {
+			GdaEntityField *field = (GdaEntityField *) list->data;
+			if (GDA_IS_QUERY_FIELD_FIELD (field)) {
+				gchar *plugin;
+
+			        g_object_get (G_OBJECT (field), "entry_plugin", &plugin, NULL);
+				if (plugin) {
+					g_object_set (G_OBJECT (params->data), "entry_plugin", plugin, NULL);
+					g_free (plugin);
+				}
+			}
+		}
+		g_slist_free (fields);
+	}
+
+	return iter;
+}
+
 static gboolean
 run_modif_query (GdaDataModelQuery *selmodel, gint query_type, GError **error)
 {
-	GdaConnection *cnc;
-	gchar *sql;
-	GdaCommand *cmd;
 	gboolean retval = FALSE;
-
-	/* check connection */
-	cnc = gda_dict_get_connection (gda_object_get_dict (GDA_OBJECT (selmodel)));
-	if (!cnc) {
-		g_set_error (error, 0, 0,
-			     _("No connection specified"));
-		return FALSE;
-	}
-
-	if (!gda_connection_is_opened (cnc)) {
-		g_set_error (error, 0, 0,
-			     _("Connection is not opened"));
-		return FALSE;
-	}
-
-	/* render the SQL and run it */
-	/*gda_object_dump (selmodel->priv->params [query_type], 0);*/
-	sql = gda_renderer_render_as_sql (GDA_RENDERER (selmodel->priv->queries[query_type]), 
-					  selmodel->priv->params [query_type],
-					  0, error);
-	if (!sql)
-		return FALSE;
-
-	g_print ("Query model SQL: %s\n", sql);
-
-	cmd = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
-	g_free (sql);
-        if (gda_connection_execute_non_query (cnc, cmd, NULL, error) >= 0)
+	GdaParameterList *plist;
+	plist = gda_query_execute (selmodel->priv->queries[query_type], 
+				   selmodel->priv->params [query_type], TRUE, error);
+	if (plist) {
 		retval = TRUE;
-	gda_command_free (cmd);
+		g_object_unref (plist);
+	}
 
 	if (retval && !selmodel->priv->defer_refresh)
 		/* do a refresh */
@@ -977,7 +1045,7 @@ run_modif_query (GdaDataModelQuery *selmodel, gint query_type, GError **error)
 }
 
 static gboolean
-gda_data_model_query_set_value_at (GdaDataModel *model, gint col, gint row, const GdaValue *value, GError **error)
+gda_data_model_query_set_value_at (GdaDataModel *model, gint col, gint row, const GValue *value, GError **error)
 {
 	GdaDataModelQuery *selmodel;
 	GdaParameterList *paramlist;
@@ -1051,7 +1119,7 @@ gda_data_model_query_set_values (GdaDataModel *model, gint row, GList *values, G
 			num = GPOINTER_TO_INT (g_object_get_data ((GObject*) params->data, "+num")) - 1;
 			if (num >= 0) {
 				/* new values */
-				GdaValue *value;
+				GValue *value;
 				value = g_list_nth_data ((GList *) values, num);
 				if (value)
 					gda_parameter_set_value (GDA_PARAMETER (params->data), value);
@@ -1102,9 +1170,12 @@ gda_data_model_query_append_values (GdaDataModel *model, const GList *values, GE
 			num = GPOINTER_TO_INT (g_object_get_data ((GObject*) params->data, "+num")) - 1;
 			if (num >= 0) {
 				/* new values only */
-				GdaValue *value;
+				GValue *value;
 				value = g_list_nth_data ((GList *) values, num);
-				gda_parameter_set_value (GDA_PARAMETER (params->data), value);
+				if (value)
+					gda_parameter_set_value (GDA_PARAMETER (params->data), value);
+				else
+					g_object_set (G_OBJECT (params->data), "use-default-value", TRUE, NULL);
 			}
 			params = g_slist_next (params);
 		}
@@ -1200,7 +1271,7 @@ gda_data_model_query_remove_row (GdaDataModel *model, gint row, GError **error)
 }
 
 static void
-gda_data_model_query_send_hint (GdaDataModel *model, GdaDataModelHint hint, const GdaValue *hint_value)
+gda_data_model_query_send_hint (GdaDataModel *model, GdaDataModelHint hint, const GValue *hint_value)
 {
 	GdaDataModelQuery *selmodel;
 	g_return_if_fail (GDA_IS_DATA_MODEL_QUERY (model));
