@@ -1,8 +1,9 @@
 /* GDA FreeTDS Provider
- * Copyright (C) 2002 The GNOME Foundation
+ * Copyright (C) 2002 - 2006 The GNOME Foundation
  *
  * AUTHORS:
  *         Holger Thon <holger.thon@gnome-db.org>
+ *         Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,13 +24,17 @@
 #endif
 
 #include <libgda/gda-data-model-array.h>
-#include <libgda/gda-intl.h>
+#include <libgda/gda-data-model-private.h>
+#include <glib/gi18n-lib.h>
 #include <libgda/gda-log.h>
 #include <glib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <tds.h>
 #include "gda-freetds.h"
+#include "gda-freetds-defs.h"
+
+#include <libgda/sql-delimiter/gda-sql-delimiter.h>
 
 #define PARENT_TYPE GDA_TYPE_SERVER_PROVIDER
 
@@ -44,7 +49,7 @@ static GObjectClass *parent_class = NULL;
 static void gda_freetds_provider_class_init (GdaFreeTDSProviderClass *klass);
 static void gda_freetds_provider_init       (GdaFreeTDSProvider      *provider,
                                              GdaFreeTDSProviderClass *klass);
-static void gda_freetds_provider_finalize   (GObject                 *object);
+static void gda_freetds_provider_finalize   (GObject *object);
 
 static gboolean gda_freetds_provider_open_connection (GdaServerProvider *provider,
                                                       GdaConnection *cnc,
@@ -58,12 +63,6 @@ static const gchar *gda_freetds_provider_get_database (GdaServerProvider *provid
 static gboolean gda_freetds_provider_change_database (GdaServerProvider *provider,
                                                       GdaConnection *cnc,
                                                       const gchar *name);
-static gboolean gda_freetds_provider_create_database_cnc (GdaServerProvider *provider,
-                                                          GdaConnection *cnc,
-                                                          const gchar *name);
-static gboolean gda_freetds_provider_drop_database_cnc (GdaServerProvider *provider,
-							GdaConnection *cnc,
-							const gchar *name);
 static GList *gda_freetds_provider_execute_command (GdaServerProvider *provider,
                                                     GdaConnection *cnc,
                                                     GdaCommand *cmd,
@@ -116,13 +115,13 @@ static gboolean gda_freetds_execute_cmd (GdaConnection *cnc, const gchar *sql);
 static int gda_freetds_provider_tds_handle_message (void *aStruct,
 						    void *bStruct,
                                                     const gboolean is_err_msg);
-#if defined(HAVE_FREETDS_VER0_6X) || defined(HAVE_FREETDS_VER0_60)
+#if defined(HAVE_FREETDS_VER0_63) || defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_60)
   static int gda_freetds_provider_tds_handle_info_msg (TDSCONTEXT *,
                                                        TDSSOCKET *,
-                                                       TDSMSGINFO *);
+                                                       _TDSMSGINFO *);
   static int gda_freetds_provider_tds_handle_err_msg (TDSCONTEXT *,
                                                       TDSSOCKET *,
-                                                      TDSMSGINFO *);
+                                                      _TDSMSGINFO *);
 #else
   static int gda_freetds_provider_tds_handle_info_msg (void *aStruct);
   static int gda_freetds_provider_tds_handle_err_msg (void *aStruct);
@@ -252,7 +251,7 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	tds_set_packet(tds_cnc->login, 512);
 
 	/* Version 0.60 api uses context additionaly */
-#if defined(HAVE_FREETDS_VER0_6X) || defined(HAVE_FREETDS_VER0_60)
+#if defined(HAVE_FREETDS_VER0_63) || defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_60)
 	tds_cnc->ctx = tds_alloc_context();
 	if (! tds_cnc->ctx) {
 		gda_log_error (_("Allocating tds context failed."));
@@ -269,7 +268,7 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	/* establish connection; change in 0.6x api */
 #if defined(HAVE_FREETDS_VER0_60)
 	tds_cnc->tds = tds_connect(tds_cnc->login, tds_cnc->ctx, NULL);
-#elif defined(HAVE_FREETDS_VER0_6X)
+#elif defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
 	tds_cnc->tds = tds_alloc_socket(tds_cnc->ctx, 512);
 	if (! tds_cnc->tds) {
 		gda_log_error (_("Allocating tds socket failed."));
@@ -297,7 +296,7 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	}
 
 	/* try to receive connection info for sanity check */
-#if defined(HAVE_FREETDS_VER0_6X)
+#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
 	/* do nothing */
 #elif defined(HAVE_FREETDS_VER0_60)
 	tds_cnc->config = tds_get_config(tds_cnc->tds, tds_cnc->login, tds_cnc->ctx->locale);
@@ -357,7 +356,7 @@ gda_freetds_free_connection_data (GdaFreeTDSConnectionData *tds_cnc)
 		tds_cnc->database = NULL;
 	}
 	if (tds_cnc->config) {
-#ifdef HAVE_FREETDS_VER0_6X
+#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
 		tds_free_connect (tds_cnc->config);
 #else
 		tds_free_config(tds_cnc->config);
@@ -370,7 +369,7 @@ gda_freetds_free_connection_data (GdaFreeTDSConnectionData *tds_cnc)
 		tds_free_socket (tds_cnc->tds);
 		tds_cnc->tds = NULL;
 	}
-#if defined(HAVE_FREETDS_VER0_6X) || defined(HAVE_FREETDS_VER0_60)
+#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63) || defined(HAVE_FREETDS_VER0_60)
 	if (tds_cnc->ctx) {
 		/* Clear callback handler */
 		tds_cnc->ctx->msg_handler = NULL;
@@ -481,32 +480,6 @@ gda_freetds_provider_change_database (GdaServerProvider *provider,
 	return ret;
 }
 
-static gboolean
-gda_freetds_provider_create_database_cnc (GdaServerProvider *provider,
-                                          GdaConnection *cnc,
-                                          const gchar *name)
-{
-	GdaFreeTDSProvider *tds_prov = (GdaFreeTDSProvider *) provider;
-
-	g_return_val_if_fail (GDA_IS_FREETDS_PROVIDER (tds_prov), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-
-	return FALSE;
-}
-
-static gboolean
-gda_freetds_provider_drop_database_cnc (GdaServerProvider *provider,
-					GdaConnection *cnc,
-					const gchar *name)
-{
-	GdaFreeTDSProvider *tds_prov = (GdaFreeTDSProvider *) provider;
-
-	g_return_val_if_fail (GDA_IS_FREETDS_PROVIDER (tds_prov), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	
-	return FALSE;
-}
-
 static GList
 *gda_freetds_provider_execute_command (GdaServerProvider *provider,
                                        GdaConnection *cnc,
@@ -528,12 +501,11 @@ static GList
 		case GDA_COMMAND_TYPE_TABLE:
 			query = g_strdup_printf ("SELECT * FROM %s", gda_command_get_text (cmd));
 			reclist = gda_freetds_provider_process_sql_commands (reclist, cnc, query);
-			if (reclist && GDA_IS_DATA_MODEL (reclist->data)) {
-				gda_data_model_set_command_text (GDA_DATA_MODEL (reclist->data),
-				                                 gda_command_get_text (cmd));
-				gda_data_model_set_command_type (GDA_DATA_MODEL (reclist->data),
-				                                 GDA_COMMAND_TYPE_TABLE);
-			}
+			if (reclist && GDA_IS_DATA_MODEL (reclist->data)) 
+				g_object_set (G_OBJECT (reclist->data), 
+					      "command_text", gda_command_get_text (cmd),
+					      "command_type", GDA_COMMAND_TYPE_TABLE, NULL);
+
 			g_free(query);
 			query = NULL;
 			break;
@@ -626,7 +598,11 @@ static const gchar
 	GdaDataModel *model = NULL;
 	
 	g_return_val_if_fail (GDA_IS_FREETDS_PROVIDER (tds_prov), NULL);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	if (cnc)
+		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	else
+		return NULL;
+
 	tds_cnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_FREETDS_HANDLE);
 	g_return_val_if_fail (tds_cnc != NULL, NULL);
 
@@ -635,11 +611,11 @@ static const gchar
 		if (model) {
 			if ((gda_data_model_get_n_columns (model) == 1)
 			    && (gda_data_model_get_n_rows (model) == 1)) {
-				GdaValue *value;
+				GValue *value;
 
-				value = (GdaValue *) gda_data_model_get_value_at
+				value = (GValue *) gda_data_model_get_value_at
 				                             (model, 0, 0);
-				tds_cnc->server_id = gda_value_stringify ((GdaValue *) value);
+				tds_cnc->server_id = gda_value_stringify ((GValue *) value);
 			}
 			g_object_unref (model);
 		}
@@ -663,9 +639,9 @@ gda_freetds_provider_get_types (GdaConnection    *cnc,
 				GdaParameterList *params)
 {
 	GdaDataModel *model = NULL;
-	TDSCOLINFO col;
-	GdaValueType gda_type;
-	GdaValue     *value = NULL;
+	_TDSCOLINFO col;
+	GType g_type;
+	GValue     *value = NULL;
 	gint i = 1;
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -678,28 +654,28 @@ gda_freetds_provider_get_types (GdaConnection    *cnc,
 		for (i = 0; i < gda_data_model_get_n_rows (model); i++) {
 			GdaRow *row = (GdaRow *) gda_data_model_get_row (model, i);
 			
-			/* first fix gda_type */
+			/* first fix g_type */
 			if (row) {
 				value = gda_row_get_value (row, 2);
-				if (gda_value_get_type (value) == GDA_VALUE_TYPE_INTEGER)
-					col.column_size = gda_value_get_integer (value);
+				if (G_VALUE_TYPE (value) == G_TYPE_INT)
+					col.column_size = g_value_get_int (value);
 				else
 					col.column_size = 0;
 				
 				value = gda_row_get_value (row, 3);
-				if (gda_value_get_type (value) != GDA_VALUE_TYPE_TINYINT)
+				if (G_VALUE_TYPE (value) != G_TYPE_CHAR)
 					col.column_type = SYBVARIANT;
 				else 
-					col.column_type = gda_value_get_tinyint (value);
+					col.column_type = g_value_get_char (value);
 				
-				gda_type = gda_freetds_get_value_type (&col);
+				g_type = gda_freetds_get_value_type (&col);
 				
 				/* col 3: type */
-				gda_value_set_gdatype (value, gda_type);
+				g_value_set_ulong (value, g_type);
 				
 				/* col 2: comment */
 				value = gda_row_get_value (row, 2);
-				gda_value_set_string (value, "");
+				g_value_set_string (value, "");
 			}
 		}
 	}
@@ -718,7 +694,10 @@ gda_freetds_provider_get_schema (GdaServerProvider *provider,
 	GdaDataModel *recset = NULL;
 	
 	g_return_val_if_fail (GDA_IS_FREETDS_PROVIDER (tds_prov), NULL);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	if (cnc)
+		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	else
+		return NULL;
 
 	switch (schema) {
 		case GDA_CONNECTION_SCHEMA_DATABASES:
@@ -775,7 +754,7 @@ gda_freetds_execute_cmd (GdaConnection *cnc, const gchar *sql)
 {
 	GdaFreeTDSConnectionData *tds_cnc;
 	GdaConnectionEvent *error;
-#ifdef HAVE_FREETDS_VER0_6X
+#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
 	int result_type = 0;
 #endif
 
@@ -795,8 +774,8 @@ gda_freetds_execute_cmd (GdaConnection *cnc, const gchar *sql)
 	}
 
 	/* there should not be any result tokens */
-#ifdef HAVE_FREETDS_VER0_6X
-	while ((tds_cnc->rc = tds_process_result_tokens (tds_cnc->tds, &tds_cnc->result_type))
+#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
+	while ((tds_cnc->rc = tds_process_result_tokens (tds_cnc->tds, &tds_cnc->result_type, NULL))
 #else
 	while ((tds_cnc->rc = tds_process_result_tokens (tds_cnc->tds)) 
 #endif
@@ -841,10 +820,10 @@ gda_freetds_execute_query (GdaConnection *cnc, const gchar* sql)
 		return NULL;
 	}
 	recset = gda_freetds_recordset_new (cnc, TRUE);
-	if (GDA_IS_FREETDS_RECORDSET (recset)) {
-		gda_data_model_set_command_text (recset, sql);
-		gda_data_model_set_command_type (recset, GDA_COMMAND_TYPE_SQL);
-	}
+	if (GDA_IS_FREETDS_RECORDSET (recset)) 
+		g_object_set (G_OBJECT (recset), 
+			      "command_text", sql,
+			      "command_type", GDA_COMMAND_TYPE_SQL, NULL);
 
 	return recset;
 }
@@ -857,19 +836,19 @@ gda_freetds_get_stringresult_of_query (GdaConnection *cnc,
                                        const gint row)
 {
 	GdaDataModel    *model = NULL;
-	GdaValue        *value = NULL;
+	GValue        *value = NULL;
 	gchar           *ret = NULL;
 
 	/* GDA_IS_CONNECTION (cnc) validation in function call */
 	model = gda_freetds_execute_query (cnc, sql);
 	
 	if (model) {
-		value = (GdaValue *) gda_data_model_get_value_at (model,
+		value = (GValue *) gda_data_model_get_value_at (model,
 		                                                  col, row);
 
 		if ((value != NULL)
-		    && (gda_value_get_type (value) != GDA_VALUE_TYPE_NULL)
-		    && (gda_value_get_type (value) != GDA_VALUE_TYPE_UNKNOWN)) {
+		    && (G_VALUE_TYPE (value) != GDA_TYPE_NULL)
+		    && (G_VALUE_TYPE (value) != G_TYPE_INVALID)) {
 			ret = (gchar *) gda_value_stringify (value);
 		}
 		
@@ -912,7 +891,7 @@ gda_freetds_get_fields (GdaConnection *cnc, GdaParameterList *params)
 	parameter = gda_parameter_list_find (params, "name");
 	g_return_val_if_fail (parameter != NULL, NULL);
 
-	table = gda_value_get_string ((GdaValue *) gda_parameter_get_value (parameter));
+	table = g_value_get_string ((GValue *) gda_parameter_get_value (parameter));
 	g_return_val_if_fail (table != NULL, NULL);
 
 	query = g_strdup_printf (TDS_SCHEMA_FIELDS, table);
@@ -941,7 +920,7 @@ static GList* gda_freetds_provider_process_sql_commands(GList         *reclist,
 	g_return_val_if_fail (tds_cnc->tds != NULL, NULL);
 
 /*	arr = gda_freetds_split_commandlist(sql); */
-	arr = g_strsplit (sql, ";", 0);
+	arr = gda_delimiter_split_sql (sql);
 	if (arr) {
 		gint n = 0;
 		while (arr[n]) {
@@ -954,11 +933,11 @@ static GList* gda_freetds_provider_process_sql_commands(GList         *reclist,
 				gda_connection_add_event (cnc, error);
 			}
 			recset = gda_freetds_recordset_new (cnc, TRUE);
-			if (GDA_IS_FREETDS_RECORDSET (recset)) {
-				gda_data_model_set_command_text (recset, arr[n]);
-				gda_data_model_set_command_type (recset, GDA_COMMAND_TYPE_SQL);
+			if (GDA_IS_FREETDS_RECORDSET (recset))
+				g_object_set (G_OBJECT (recset), 
+					      "command_text", arr[n],
+					      "command_type", GDA_COMMAND_TYPE_SQL, NULL);
 				reclist = g_list_append (reclist, recset);
-                        }
 
                         n++;
 
@@ -980,20 +959,37 @@ gda_freetds_provider_class_init (GdaFreeTDSProviderClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_freetds_provider_finalize;
+
+	provider_class->get_version = gda_freetds_provider_get_version;
+	provider_class->get_server_version = gda_freetds_provider_get_server_version;
+	provider_class->get_info = NULL;
+	provider_class->supports_feature = gda_freetds_provider_supports;
+	provider_class->get_schema = gda_freetds_provider_get_schema;
+
+	provider_class->get_data_handler = NULL;
+	provider_class->string_to_value = NULL;
+	provider_class->get_def_dbms_type = NULL;
+
 	provider_class->open_connection = gda_freetds_provider_open_connection;
 	provider_class->close_connection = gda_freetds_provider_close_connection;
 	provider_class->get_database = gda_freetds_provider_get_database;
 	provider_class->change_database = gda_freetds_provider_change_database;
-	provider_class->create_database_cnc = gda_freetds_provider_create_database_cnc;
-	provider_class->drop_database_cnc = gda_freetds_provider_drop_database_cnc;
+
+	provider_class->supports_operation = NULL;
+        provider_class->create_operation = NULL;
+        provider_class->render_operation = NULL;
+        provider_class->perform_operation = NULL;
+
 	provider_class->execute_command = gda_freetds_provider_execute_command;
+	provider_class->get_last_insert_id = NULL;
+
 	provider_class->begin_transaction = gda_freetds_provider_begin_transaction;
 	provider_class->commit_transaction = gda_freetds_provider_commit_transaction;
 	provider_class->rollback_transaction = gda_freetds_provider_rollback_transaction;
-	provider_class->get_version = gda_freetds_provider_get_version;
-	provider_class->get_server_version = gda_freetds_provider_get_server_version;
-	provider_class->supports = gda_freetds_provider_supports;
-	provider_class->get_schema = gda_freetds_provider_get_schema;
+	
+	provider_class->create_blob = NULL;
+	provider_class->fetch_blob = NULL;	
+	
 
 #ifdef HAVE_FREETDS_VER0_5X
 	if (tds_cbs_initialized == FALSE) {
@@ -1036,7 +1032,7 @@ static int gda_freetds_provider_tds_handle_message (void *aStruct,
                                                     const gboolean is_err_msg)
 {
 	TDSSOCKET *tds = (TDSSOCKET *) aStruct;
-	TDSMSGINFO *msg_info = (TDSMSGINFO *) bStruct;
+	_TDSMSGINFO *msg_info = (_TDSMSGINFO *) bStruct;
 	GdaConnection *cnc = NULL;
 	GdaFreeTDSConnectionData *tds_cnc = NULL;
 	GdaConnectionEvent *error = NULL;
@@ -1088,14 +1084,14 @@ static int gda_freetds_provider_tds_handle_message (void *aStruct,
 	return TDS_SUCCEED;
 }
 
-#if defined(HAVE_FREETDS_VER0_6X) || defined(HAVE_FREETDS_VER0_60)
+#if defined(HAVE_FREETDS_VER0_63) || defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_60)
 /* FIXME: rewrite tds_handle_message as well/use new parameters here */
 static int
 gda_freetds_provider_tds_handle_info_msg (TDSCONTEXT *ctx, TDSSOCKET *tds,
-                                          TDSMSGINFO *msg)
+                                          _TDSMSGINFO *msg)
 {
 	return gda_freetds_provider_tds_handle_message ((void *) tds,
-#ifdef HAVE_FREETDS_VER0_6X
+#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
 							(void *) msg,
 #else
 							(void *) tds->msg_info,
@@ -1110,14 +1106,14 @@ gda_freetds_provider_tds_handle_info_msg (void *aStruct)
 }
 #endif
 
-#if defined(HAVE_FREETDS_VER0_6X) || defined(HAVE_FREETDS_VER0_60)
+#if defined(HAVE_FREETDS_VER0_63) || defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_60)
 /* FIXME: rewrite tds_handle_message as well/use new parameters here */
 static int
 gda_freetds_provider_tds_handle_err_msg (TDSCONTEXT *ctx, TDSSOCKET *tds,
-                                         TDSMSGINFO *msg)
+                                         _TDSMSGINFO *msg)
 {
 	return gda_freetds_provider_tds_handle_message ((void *) tds,
-#ifdef HAVE_FREETDS_VER0_6X
+#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
 							(void *) msg,
 #else
 							(void *) tds->msg_info,
